==> arangodb_repository-0.9.0.tar.gz: lib/src/arangodb_search.dart <==
import 'dart:async';

import 'package:arangodb_repository/arangodb_repository.dart';
import 'package:nosql_repository/nosql_repository.dart';
import 'package:tuple/tuple.dart';

class ArangoDbSearch<TEntity> extends RepositorySearch<TEntity> {
  final ArangoDbRepository repository;

  ArangoDbSearch(this.repository);

  @override
  Future<SearchResult> searchWithCount(
    SearchCriteria criteria,
    DbPrincipal principal, {
    String? permission,
  }) async {
    var filters = _createFilters(criteria);
    var filterQuery = filters.item1;
    var parameters = filters.item2;
    var count = await (_getCount(
      filterQuery,
      {...parameters},
      principal,
      permission: permission!,
    ));

    var page =
        _createPage(filterQuery, criteria, parameters, principal, permission);

    var ret = SearchResult(page, count);
    return ret;
  }

  Stream<Map<String, dynamic>> _createPage(
      String filterQuery,
      SearchCriteria criteria,
      Map<String, dynamic> parameters,
      DbPrincipal principal,
      String permission) {
    final queryBuffer = StringBuffer();

    queryBuffer.writeln('FOR entity in entities ');
    queryBuffer.writeln(filterQuery);

    var sort = _createSort(criteria);
    queryBuffer.writeln(sort);

    if (criteria.skip != null || criteria.take != null) {
      queryBuffer.writeln('LIMIT @skip, @take');
      parameters.addAll({'skip': criteria.skip, 'take': criteria.take});
    }

    var retrn = _createReturn(criteria);
    queryBuffer.writeln(retrn);

    var query = queryBuffer.toString();

    var page = repository.runQuery(query, principal,
        parameters: parameters,
        permission: permission,
        collectionAlias: 'entities');
    return page;
  }

  Tuple2<String, Map<String, dynamic>> _createFilters(SearchCriteria criteria) {
    throw UnimplementedError();

    // var context = Context();
    // var buffer = StringBuffer();
    // for (var condition in (criteria.searchConditions ??
    //    <List<Expression>>[])) {
    //   var expression = condition.render(context);
    //   buffer.writeln('FILTER $expression');
    // }
    // var ret = Tuple2(buffer.toString(), context.parameters);
    // return ret;
  }

  String _createSort(SearchCriteria criteria) {
    var sortBuffer = StringBuffer();
    for (var f in criteria.orderByFields ?? []) {
      if (sortBuffer.isEmpty) {
        sortBuffer.write(sortBuffer.isEmpty ? 'sort' : ',');
      }
      sortBuffer.write(' entity.${_sanitizePath(f.path)}');
    }

    return sortBuffer.toString();
  }

  String _createReturn(SearchCriteria criteria) {
    if ((criteria.returnFields ?? []).isEmpty) {
      return 'RETURN entity';
    }

    var sb = StringBuffer();

    for (var rf in criteria.returnFields!) {
      if (sb.isEmpty) {
        sb.write('RETURN {');
      } else {
        sb.write(', ');
      }

      var alias = rf.alias ?? rf.path!.replaceAll('.', '_');
      sb.write(' $alias : entity.${rf.path} ');
    }
    sb.write('}');

    var ret = sb.toString();
    return ret;
  }

  String _sanitizePath(String path) {
    var sanitized = path.replaceAll(RegExp(r'[^a-zA-Z0-9\.\_]'), '');
    return sanitized;
  }

  Future<int> _getCount(
      String query, Map<String, dynamic> parameters, DbPrincipal principal,
      {required String permission}) async {
    var countQuery = 'RETURN { cnt: length($query) }';
    var count = (await repository
        .runQuery(countQuery, principal,
            parameters: parameters,
            permission: permission,
            collectionAlias: 'entities')
        .first)['cnt'] as int;

    return count;
  }

  @override
  Future<int> count(SearchCriteria criteria, DbPrincipal principal,
      {String? permission}) async {
    var filters = _createFilters(criteria);
    var baseQuery = filters.item1;
    var parameters = filters.item2;
    var count = await _getCount(
      baseQuery,
      parameters,
      principal,
      permission: permission!,
    );

    return count;
  }

  @override
  Stream<Map<String, dynamic>> search(
    SearchCriteria criteria,
    DbPrincipal principal, {
    String? permission,
  }) {
    var filters = _createFilters(criteria);
    var filterQuery = filters.item1;
    var parameters = filters.item2;

    var page =
        _createPage(filterQuery, criteria, parameters, principal, permission!);

    return page;
  }
}

==> arangodb_repository-1.0.0-rc.1.tar.gz: lib/src/arangodb_search.dart <==
import 'dart:async';

import 'package:arangodb_repository/arangodb_repository.dart';
import 'package:nosql_repository/nosql_repository.dart';
import 'package:tuple/tuple.dart';

import 'expression_rendering/context.dart';
import 'expression_rendering/expression_extension.dart';

class ArangoDbSearch<TEntity> extends RepositorySearch<TEntity> {
  final ArangoDbRepository repository;

  ArangoDbSearch(this.repository);

  @override
  Future<SearchResult> searchWithCount(
    SearchCriteria criteria,
    DbPrincipal principal, {
    String permission = '',
  }) async {
    var filters = _createFilters(criteria);
    var filterQuery = filters.item1;
    var parameters = filters.item2;
    var count = await (_getCount(
      filterQuery,
      {...parameters},
      principal,
      permission: permission,
    ));

    var page = _createPage(
      filterQuery,
      criteria,
      parameters,
      principal,
      permission,
    );

    var ret = SearchResult(page, count);
    return ret;
  }

  Stream<Map<String, dynamic>> _createPage(
    String filterQuery,
    SearchCriteria criteria,
    Map<String, dynamic> parameters,
    DbPrincipal principal,
    String permission,
  ) {
    final queryBuffer = StringBuffer();

    queryBuffer.writeln('FOR entity in entities ');
    queryBuffer.writeln(filterQuery);

    var sort = _createSort(criteria);
    queryBuffer.writeln(sort);

    if (criteria.skip != null || criteria.take != null) {
      queryBuffer.writeln('LIMIT @skip, @take');
      parameters.addAll({'skip': criteria.skip, 'take': criteria.take});
    }

    var retrn = _createReturn(criteria);
    queryBuffer.writeln(retrn);

    var query = queryBuffer.toString();

    var page = repository.runQuery(query, principal,
        parameters: parameters,
        permission: permission,
        collectionAlias: 'entities');
    return page;
  }

  Tuple2<String, Map<String, dynamic>> _createFilters(SearchCriteria criteria) {
    var context = Context();
    var buffer = StringBuffer();
    for (var condition in criteria.searchConditions) {
      var expression = condition.render(context);
      buffer.writeln('FILTER $expression');
    }
    var ret = Tuple2(buffer.toString(), context.parameters);
    return ret;
  }

  String _createSort(SearchCriteria criteria) {
    var sortBuffer = StringBuffer();
    for (var f in criteria.orderByFields) {
      if (sortBuffer.isEmpty) {
        sortBuffer.write(sortBuffer.isEmpty ? 'sort' : ',');
      }
      sortBuffer.write(' entity.${_sanitizePath(f.path)}');
    }

    return sortBuffer.toString();
  }

  String _createReturn(SearchCriteria criteria) {
    if ((criteria.returnFields).isEmpty) {
      return 'RETURN entity';
    }

    var sb = StringBuffer();

    for (var rf in criteria.returnFields) {
      if (sb.isEmpty) {
        sb.write('RETURN {');
      } else {
        sb.write(', ');
      }

      var alias = rf.alias ?? rf.path!.replaceAll('.', '_');
      sb.write(' $alias : entity.${rf.path} ');
    }
    sb.write('}');

    var ret = sb.toString();
    return ret;
  }

  String _sanitizePath(String path) {
    var sanitized = path.replaceAll(RegExp(r'[^a-zA-Z0-9\.\_]'), '');
    return sanitized;
  }

  Future<int> _getCount(
    String query,
    Map<String, dynamic> parameters,
    DbPrincipal principal, {
    String permission = '',
  }) async {
    var countQuery = 'RETURN { cnt: length($query) }';
    var count = (await repository
        .runQuery(countQuery, principal,
            parameters: parameters,
            permission: permission,
            collectionAlias: 'entities')
        .first)['cnt'] as int;

    return count;
  }

  @override
  Future<int> count(
    SearchCriteria criteria,
    DbPrincipal principal, {
    String permission = '',
  }) async {
    var filters = _createFilters(criteria);
    var baseQuery = filters.item1;
    var parameters = filters.item2;
    var count = await _getCount(
      baseQuery,
      parameters,
      principal,
      permission: permission,
    );

    return count;
  }

  @override
  Stream<Map<String, dynamic>> search(
    SearchCriteria criteria,
    DbPrincipal principal, {
    String permission = '',
  }) {
    var filters = _createFilters(criteria);
    var filterQuery = filters.item1;
    var parameters = filters.item2;

    var page =
        _createPage(filterQuery, criteria, parameters, principal, permission);

    return page;
  }
}
